@@ -5,7 +5,7 @@ module Database.Redis (
     --
     -- @
     -- -- connects to localhost:6379
-    -- conn <- 'connect' 'defaultConnectInfo'
+    -- conn <- 'checkedConnect' 'defaultConnectInfo'
     -- @
     --
     -- Send commands to the server:
@@ -101,8 +101,9 @@ module Database.Redis (
     --  Automatic pipelining makes use of Haskell's laziness. As long as a
     --  previous reply is not evaluated, subsequent commands can be pipelined.
     --
-    --  Automatic pipelining also works across several calls to 'runRedis', as
-    --  long as replies are only evaluated /outside/ the 'runRedis' block.
+    --  Automatic pipelining is limited to the scope of 'runRedis' call and
+    --  it is guaranteed that every reply expected as a part of 'runRedis'
+    --  execution gets received after 'runRedis` invocation.
     --
     --  To keep memory usage low, the number of requests \"in the pipeline\" is
     --  limited (per connection) to 1000. After that number, the next command is
@@ -121,6 +122,14 @@ module Database.Redis (
     --    functions throw a 'ConnectionLostException'. It can only be caught
     --    outside of 'runRedis'.
     --
+    --  [Trying to connect to an unreachable server:] When trying to connect to
+    --    a server that does not exist or can't be reached, the connection pool
+    --    only starts the first connection when actually executing a call to
+    --    the server. This can lead to discovering very late that the server is
+    --    not available, for example when running a server that logs to Redis.
+    --    To prevent this, run a 'ping' command directly after connecting or
+    --    use the 'checkedConnect' function which encapsulates this behavior.
+    --
     --  [Exceptions:] Any exceptions can only be caught /outside/ of 'runRedis'.
     --    This way the connection pool can properly close the connection, making
     --    sure it is not left in an unusable state, e.g. closed or inside a
@@ -129,10 +138,11 @@ module Database.Redis (
     
     -- * The Redis Monad
     Redis(), runRedis,
-    RedisCtx(), MonadRedis(),
+    unRedis, reRedis,
+    RedisCtx(), MonadRedis(..),
 
     -- * Connection
-    Connection, connect,
+    Connection, connect, checkedConnect,
     ConnectInfo(..),defaultConnectInfo,
     HostName,PortID(..),
     
