@@ -1,19 +1,22 @@
 {-# LANGUAGE OverloadedStrings, GeneralizedNewtypeDeriving, RecordWildCards,
-    MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances #-}
+    MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances, CPP #-}
 
 module Database.Redis.Core (
-    Connection, connect,
+    Connection(..), connect, checkedConnect,
     ConnectInfo(..), defaultConnectInfo,
-    Redis(),runRedis,
+    Redis(), runRedis, unRedis, reRedis,
     RedisCtx(..), MonadRedis(..),
     send, recv, sendRequest,
-    auth, select
+    auth, select, ping
 ) where
 
 import Prelude
+#if __GLASGOW_HASKELL__ < 710
 import Control.Applicative
+#endif
 import Control.Monad.Reader
 import qualified Data.ByteString as B
+import Data.IORef
 import Data.Pool
 import Data.Time
 import Network
@@ -32,9 +35,11 @@ import Database.Redis.Types
 --
 --  In this context, each result is wrapped in an 'Either' to account for the
 --  possibility of Redis returning an 'Error' reply.
-newtype Redis a = Redis (ReaderT (PP.Connection Reply) IO a)
+newtype Redis a = Redis (ReaderT RedisEnv IO a)
     deriving (Monad, MonadIO, Functor, Applicative)
 
+data RedisEnv = Env { envConn :: PP.Connection, envLastReply :: IORef Reply }
+
 -- |This class captures the following behaviour: In a context @m@, a command
 --  will return it's result wrapped in a \"container\" of type @f@.
 --
@@ -59,20 +64,48 @@ instance MonadRedis Redis where
 --  while all connections from the pool are in use.
 runRedis :: Connection -> Redis a -> IO a
 runRedis (Conn pool) redis =
-    withResource pool $ \conn -> runRedisInternal conn redis
+  withResource pool $ \conn -> runRedisInternal conn redis
+
+-- |Deconstruct Redis constructor.
+--
+--  'unRedis' and 'reRedis' can be used to define instances for
+--  arbitrary typeclasses.
+-- 
+--  WARNING! These functions are considered internal and no guarantee
+--  is given at this point that they will not break in future.
+unRedis :: Redis a -> ReaderT RedisEnv IO a
+unRedis (Redis r) = r
+
+-- |Reconstruct Redis constructor.
+reRedis :: ReaderT RedisEnv IO a -> Redis a
+reRedis r = Redis r
 
 -- |Internal version of 'runRedis' that does not depend on the 'Connection'
---  abstraction. Used to run the AUTH command when connecting. 
-runRedisInternal :: PP.Connection Reply -> Redis a -> IO a
-runRedisInternal env (Redis redis) = runReaderT redis env
+--  abstraction. Used to run the AUTH command when connecting.
+runRedisInternal :: PP.Connection -> Redis a -> IO a
+runRedisInternal conn (Redis redis) = do
+  -- Dummy reply in case no request is sent.
+  ref <- newIORef (SingleLine "nobody will ever see this")
+  r <- runReaderT redis (Env conn ref)
+  -- Evaluate last reply to keep lazy IO inside runRedis.
+  readIORef ref >>= (`seq` return ())
+  return r
 
+setLastReply :: Reply -> ReaderT RedisEnv IO ()
+setLastReply r = do
+  ref <- asks envLastReply
+  lift (writeIORef ref r)
 
 recv :: (MonadRedis m) => m Reply
-recv = liftRedis $ Redis $ ask >>= liftIO . PP.recv
+recv = liftRedis $ Redis $ do
+  conn <- asks envConn
+  r <- liftIO (PP.recv conn)
+  setLastReply r
+  return r
 
 send :: (MonadRedis m) => [B.ByteString] -> m ()
 send req = liftRedis $ Redis $ do
-    conn <- ask
+    conn <- asks envConn
     liftIO $ PP.send conn (renderRequest req)
 
 -- |'sendRequest' can be used to implement commands from experimental
@@ -88,10 +121,12 @@ send req = liftRedis $ Redis $ do
 sendRequest :: (RedisCtx m f, RedisResult a)
     => [B.ByteString] -> m (f a)
 sendRequest req = do
-    r <- liftRedis $ Redis $ do
-        conn <- ask
-        liftIO $ PP.request conn (renderRequest req)
-    returnDecode r
+    r' <- liftRedis $ Redis $ do
+        conn <- asks envConn
+        r <- liftIO $ PP.request conn (renderRequest req)
+        setLastReply r
+        return r
+    returnDecode r'
 
 
 --------------------------------------------------------------------------------
@@ -100,7 +135,7 @@ sendRequest req = do
 
 -- |A threadsafe pool of network connections to a Redis server. Use the
 --  'connect' function to create one.
-newtype Connection = Conn (Pool (PP.Connection Reply))
+newtype Connection = Conn (Pool PP.Connection)
 
 -- |Information for connnecting to a Redis server.
 --
@@ -108,7 +143,7 @@ newtype Connection = Conn (Pool (PP.Connection Reply))
 -- Instead use 'defaultConnectInfo' and update it with record syntax. For
 -- example to connect to a password protected Redis server running on localhost
 -- and listening to the default port:
--- 
+--
 -- @
 -- myConnectInfo :: ConnectInfo
 -- myConnectInfo = defaultConnectInfo {connectAuth = Just \"secret\"}
@@ -154,14 +189,15 @@ defaultConnectInfo = ConnInfo
     , connectMaxIdleTime    = 30
     }
 
--- |Opens a 'Connection' to a Redis server designated by the given
---  'ConnectInfo'.
+-- |Constructs a 'Connection' pool to a Redis server designated by the 
+--  given 'ConnectInfo'. The first connection is not actually established
+--  until the first call to the server.
 connect :: ConnectInfo -> IO Connection
 connect ConnInfo{..} = Conn <$>
     createPool create destroy 1 connectMaxIdleTime connectMaxConnections
   where
     create = do
-        conn <- PP.connect connectHost connectPort reply
+        conn <- PP.connect connectHost connectPort
         runRedisInternal conn $ do
             -- AUTH
             case connectAuth of
@@ -173,6 +209,16 @@ connect ConnInfo{..} = Conn <$>
 
     destroy = PP.disconnect
 
+-- |Constructs a 'Connection' pool to a Redis server designated by the
+--  given 'ConnectInfo', then tests if the server is actually there. 
+--  Throws an exception if the connection to the Redis server can't be
+--  established.
+checkedConnect :: ConnectInfo -> IO Connection
+checkedConnect connInfo = do
+    conn <- connect connInfo
+    runRedis conn $ void ping
+    return conn
+
 -- The AUTH command. It has to be here because it is used in 'connect'.
 auth
     :: B.ByteString -- ^ password
@@ -184,4 +230,10 @@ select
     :: RedisCtx m f
     => Integer -- ^ index
     -> m (f Status)
-select ix = sendRequest ["SELECT", encode ix]
\ No newline at end of file
+select ix = sendRequest ["SELECT", encode ix]
+
+-- The PING command. Used in 'checkedConnect'.
+ping
+    :: (RedisCtx m f)
+    => m (f Status)
+ping  = sendRequest (["PING"] )
